//+--------------------------------------------------------------------------------------------+
//|                                                                  #MTF_FractalsBreakout.mq4 |
//|                                                             Copyright  2010, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2010, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//----   --------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 0

//----  ------------------------------------------------------------------------------+
#define ObjName_Signature         "#MTF_FractalsBreakout"
#define AlertMessage_Upper        ": breakout the UPPER line."
#define AlertMessage_Lower        ": breakout the LOWER line."
#define ArrowsCode                3

//----   ----------------------------------------------------------------------+
extern int    TimeFrame                       = 0;
extern int    MinimalDistance                 = 0;
extern int    MaxBarsBetweenFractals          = 0;
extern bool   OnlyByTwoLastFractals           = false;
extern bool   PopUpAlert                      = false;
extern bool   PopUpAlert_ByCurrentTick        = false;
extern color  Color_UpperFractals             = Gold;
extern color  Color_UpperLine                 = Gold;
extern color  Color_LowerFractals             = DarkViolet;
extern color  Color_LowerLine                 = DarkViolet;
extern int    Line_Style                      = STYLE_SOLID;
extern int    Line_Width                      = 1;

//----  --------------------------------------------------------------------------------+

//-----   -----------------------------------------------------------------+
static int      period;
static int      alert_barshift;
static datetime bartime;
static datetime bartime_alert_buy;
static datetime bartime_alert_sell;
static double   minimal_distance;
static string   short_name;
static string   objname_upper_line;
static string   objname_upper_last_fr;
static string   objname_upper_prev_fr;
static string   objname_lower_line;
static string   objname_lower_last_fr;
static string   objname_lower_prev_fr;

//---------------------------------------------------------------------------------------------+
//---- init -----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   bartime = 0;
   //----   --------------------------------------------------------------+
   if(!PopUpAlert_ByCurrentTick) alert_barshift = 1;
   else alert_barshift = 0;
   if(Digits < 4) double point = 0.01;
   else point = 0.0001;
   minimal_distance = NormalizeDouble(MinimalDistance * point,Digits);
   period = Period();
   if(TimeFrame <= 0) TimeFrame = period;
   objname_upper_line    = StringConcatenate(ObjName_Signature,"_upper_line");
   objname_upper_last_fr = StringConcatenate(ObjName_Signature,"_upper_last_fr");
   objname_upper_prev_fr = StringConcatenate(ObjName_Signature,"_upper_prev_fr");
   objname_lower_line    = StringConcatenate(ObjName_Signature,"_lower_line");
   objname_lower_last_fr = StringConcatenate(ObjName_Signature,"_lower_last_fr");
   objname_lower_prev_fr = StringConcatenate(ObjName_Signature,"_lower_prev_fr");
   //----  ""   ----------------------------------------------+
   short_name = StringConcatenate("#MTF_FractalsBreakout(",TimeFrameToStr(TimeFrame),")");
   IndicatorShortName(short_name);
   //----   -----------------------------------------------------------------+
   return(0);
   Print("---- Programming by Serega Lykov, http://mtexperts.narod.ru/ ----");
  }

//---------------------------------------------------------------------------------------------+
//---- deinit ---------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   DeleteObjects(ObjName_Signature);
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- start ----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   datetime cur_bartime = iTime(NULL,TimeFrame,0);
   if(bartime == cur_bartime) return(0);
   bartime = cur_bartime;
   int bars = iBars(NULL,TimeFrame);
   bool is_upper_line = false;
   bool is_lower_line = false;
   for(int i=3; i<=bars; i++)
     {
      double last_upper_fr = iFractals(NULL,TimeFrame,MODE_UPPER,i);
      if(last_upper_fr == 0.0) continue;
      last_upper_fr = NormalizeDouble(last_upper_fr,Digits);
      int last_upper_fr_barshift = i;
      if(MaxBarsBetweenFractals > 0) int limit = i + MaxBarsBetweenFractals + 1;
      else limit = bars;
      for(int k=i+1; k<=limit; k++)
        {
         double prev_upper_fr = iFractals(NULL,TimeFrame,MODE_UPPER,k);
         if(prev_upper_fr != 0.0)
           {
            prev_upper_fr = NormalizeDouble(prev_upper_fr,Digits);
            is_upper_line = NormalizeDouble(prev_upper_fr - last_upper_fr,Digits) >= minimal_distance;
            if(OnlyByTwoLastFractals || is_upper_line)
              {
               int prev_upper_fr_barshift = k;
               break;
              }
           }
        }
      break;
     }
   for(i=3; i<=bars; i++)
     {
      double last_lower_fr = iFractals(NULL,TimeFrame,MODE_LOWER,i);
      if(last_lower_fr == 0.0) continue;
      last_lower_fr = NormalizeDouble(last_lower_fr,Digits);
      int last_lower_fr_barshift = i;
      if(MaxBarsBetweenFractals > 0) limit = i + MaxBarsBetweenFractals + 1;
      else limit = bars;
      for(k=i+1; k<=limit; k++)
        {
         double prev_lower_fr = iFractals(NULL,TimeFrame,MODE_LOWER,k);
         if(prev_lower_fr != 0.0)
           {
            prev_lower_fr = NormalizeDouble(prev_lower_fr,Digits);
            is_lower_line = NormalizeDouble(last_lower_fr - prev_lower_fr,Digits) >= minimal_distance;
            if(OnlyByTwoLastFractals || is_lower_line)
              {
               int prev_lower_fr_barshift = k;
               break;
              }
           }
        }
      break;
     }
   if(TimeFrame == period)
     {
      datetime last_upper_fr_bartime = Time[last_upper_fr_barshift];
      datetime prev_upper_fr_bartime = Time[prev_upper_fr_barshift];
      datetime last_lower_fr_bartime = Time[last_lower_fr_barshift];
      datetime prev_lower_fr_bartime = Time[prev_lower_fr_barshift];
     }
   else
     {
      if(TimeFrame < period)
        {
         last_upper_fr_bartime = Time[iBarShift(NULL,0,iTime(NULL,TimeFrame,last_upper_fr_barshift))];
         prev_upper_fr_bartime = Time[iBarShift(NULL,0,iTime(NULL,TimeFrame,prev_upper_fr_barshift))];
         last_lower_fr_bartime = Time[iBarShift(NULL,0,iTime(NULL,TimeFrame,last_lower_fr_barshift))];
         prev_lower_fr_bartime = Time[iBarShift(NULL,0,iTime(NULL,TimeFrame,prev_lower_fr_barshift))];
        }
      else
        {
         for(i=iBarShift(NULL,0,iTime(NULL,TimeFrame,last_upper_fr_barshift)); i>=0; i--)
           {
            if(NormalizeDouble(High[i],Digits) == last_upper_fr)
              {
               last_upper_fr_bartime = Time[i];
               break;
              }
           }
         for(i=iBarShift(NULL,0,iTime(NULL,TimeFrame,prev_upper_fr_barshift)); i>=0; i--)
           {
            if(NormalizeDouble(High[i],Digits) == prev_upper_fr)
              {
               prev_upper_fr_bartime = Time[i];
               break;
              }
           }
         for(i=iBarShift(NULL,0,iTime(NULL,TimeFrame,last_lower_fr_barshift)); i>=0; i--)
           {
            if(NormalizeDouble(Low[i],Digits) == last_lower_fr)
              {
               last_lower_fr_bartime = Time[i];
               break;
              }
           }
         for(i=iBarShift(NULL,0,iTime(NULL,TimeFrame,prev_lower_fr_barshift)); i>=0; i--)
           {
            if(NormalizeDouble(Low[i],Digits) == prev_lower_fr)
              {
               prev_lower_fr_bartime = Time[i];
               break;
              }
           }
        }
     }
   if(is_upper_line)
     {
      CreateObjectArrow(objname_upper_prev_fr,prev_upper_fr_bartime,prev_upper_fr,ArrowsCode,Color_UpperFractals);
      CreateObjectArrow(objname_upper_last_fr,last_upper_fr_bartime,last_upper_fr,ArrowsCode,Color_UpperFractals);
      CreateObjectTrendLine(objname_upper_line,prev_upper_fr_bartime,prev_upper_fr,last_upper_fr_bartime,last_upper_fr,Line_Style,Line_Width,Color_UpperLine);
     }
   else
     {
      if(ObjectFind(objname_upper_prev_fr) >= 0) ObjectDelete(objname_upper_prev_fr);
      if(ObjectFind(objname_upper_last_fr) >= 0) ObjectDelete(objname_upper_last_fr);
      if(ObjectFind(objname_upper_line)    >= 0) ObjectDelete(objname_upper_line);
     }
   if(is_lower_line)
     {
      CreateObjectArrow(objname_lower_prev_fr,prev_lower_fr_bartime,prev_lower_fr,ArrowsCode,Color_LowerFractals);
      CreateObjectArrow(objname_lower_last_fr,last_lower_fr_bartime,last_lower_fr,ArrowsCode,Color_LowerFractals);
      CreateObjectTrendLine(objname_lower_line,prev_lower_fr_bartime,prev_lower_fr,last_lower_fr_bartime,last_lower_fr,Line_Style,Line_Width,Color_LowerLine);
     }
   else
     {
      if(ObjectFind(objname_lower_prev_fr) >= 0) ObjectDelete(objname_lower_prev_fr);
      if(ObjectFind(objname_lower_last_fr) >= 0) ObjectDelete(objname_lower_last_fr);
      if(ObjectFind(objname_lower_line)    >= 0) ObjectDelete(objname_lower_line);
     }
   if(PopUpAlert)
     {
      double curr_close = NormalizeDouble(Close[alert_barshift],  Digits);
      double prev_close = NormalizeDouble(Close[alert_barshift+1],Digits);
      if(is_upper_line)
        {
         if(bartime_alert_buy != Time[alert_barshift])
           {
            if(curr_close  > NormalizeDouble(ObjectGetValueByShift(objname_upper_line,alert_barshift),  Digits) &&
               prev_close <= NormalizeDouble(ObjectGetValueByShift(objname_upper_line,alert_barshift+1),Digits))
              {
               Alert(StringConcatenate(short_name,AlertMessage_Upper));
               bartime_alert_buy = Time[alert_barshift];
              }
           }
        }
      if(is_lower_line)
        {
         if(bartime_alert_sell != Time[alert_barshift])
           {
            if(curr_close  < NormalizeDouble(ObjectGetValueByShift(objname_lower_line,alert_barshift),  Digits) &&
               prev_close >= NormalizeDouble(ObjectGetValueByShift(objname_lower_line,alert_barshift+1),Digits))
              {
               Alert(StringConcatenate(short_name,AlertMessage_Lower));
               bartime_alert_sell = Time[alert_barshift];
              }
           }
        }
     }
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- TimeFrameToStr -------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string TimeFrameToStr(int period)
  {
   switch(period)
     {
      case     0: string TimeFrameStr = TimeFrameToStr(Period());
      case     1:        TimeFrameStr = "M1";          break;
      case     5:        TimeFrameStr = "M5";          break;
      case    15:        TimeFrameStr = "M15";         break;
      case    30:        TimeFrameStr = "M30";         break;
      case    60:        TimeFrameStr = "H1";          break;
      case   240:        TimeFrameStr = "H4";          break;
      case  1440:        TimeFrameStr = "D1";          break;
      case 10080:        TimeFrameStr = "W1";          break;
      case 43200:        TimeFrameStr = "MN";          break;
      default:           TimeFrameStr = StringConcatenate("M",DoubleToStr(period,0));
     }
   return(TimeFrameStr);
  }

//---------------------------------------------------------------------------------------------+
//---- CreateObjectArrow ----------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void CreateObjectArrow(string obj_name, datetime obj_time1, double obj_price1, int obg_arrowcode, color obj_color, int obj_window = 0)
  {
   if(ObjectFind(obj_name) < 0) ObjectCreate(obj_name,OBJ_ARROW,obj_window,obj_time1,obj_price1);
   else
     {
      if(ObjectGet(obj_name,OBJPROP_TIME1) != obj_time1 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits) != obj_price1)
        {
         ObjectMove(obj_name,0,obj_time1,obj_price1);
        }
     }
   ObjectSet(obj_name,OBJPROP_ARROWCODE,obg_arrowcode);
   ObjectSet(obj_name,OBJPROP_COLOR,obj_color);
  }

//---------------------------------------------------------------------------------------------+
//---- CreateObjectTrendLine ------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void CreateObjectTrendLine(string obj_name, datetime obj_time1, double obj_price1, datetime obj_time2, double obj_price2, int obj_style, int obj_width, color obj_color, bool obj_ray = true, bool obj_back = false, int obj_window = 0)
  {
  	if(ObjectFind(obj_name) < 0) ObjectCreate(obj_name,OBJ_TREND,obj_window,obj_time1,obj_price1,obj_time2,obj_price2);
  	else
  	  {
      if(ObjectGet(obj_name,OBJPROP_TIME1) != obj_time1 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits) != obj_price1)
        {
         ObjectMove(obj_name,0,obj_time1,obj_price1);
        }
      if(ObjectGet(obj_name,OBJPROP_TIME2) != obj_time2 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE2),Digits) != obj_price2)
        {
         ObjectMove(obj_name,1,obj_time2,obj_price2);
        }
  	  }
  	ObjectSet(obj_name,OBJPROP_COLOR,obj_color); 
   ObjectSet(obj_name,OBJPROP_STYLE,obj_style);
   ObjectSet(obj_name,OBJPROP_WIDTH,obj_width);
   ObjectSet(obj_name,OBJPROP_RAY,  obj_ray);
   ObjectSet(obj_name,OBJPROP_BACK, obj_back);
  }

//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_name_signature, int obj_type=-1)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      if(obj_type >= 0) if(ObjectType(obj_name) != obj_type) continue;
      if(StringFind(obj_name,obj_name_signature,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }

//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+